@@ -1,12 +1,9 @@
 // #![deny(missing_docs)]
 
 pub mod brb_membership;
+pub mod ed25519;
 pub use crate::brb_membership::{Ballot, Generation, Reconfig, State, Vote, VoteMsg};
+pub use crate::ed25519::{PublicKey, SecretKey, Signature};
 
 pub mod error;
 pub use crate::error::Error;
-
-pub mod actor;
-pub use actor::{Actor, Sig, SigningActor};
-
-pub use signature;
